<?php

namespace Drupal\custom_term_cleanup\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form for managing term deletions.
 */
class TermCleanupForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Database connection.
   */
  protected Connection $database;

  /**
   * Constructs a new BulkUpdateUserForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Database\Connection $database
   *   Database connection.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, Connection $database) {
    $this->entityTypeManager = $entity_type_manager;
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
     $container->get('entity_type.manager'),
     $container->get('database'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'custom_term_cleanup_term_cleanup_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['select_all'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Select All'),
      '#attributes' => ['class' => ['select-all']],
      '#attached' => ['library' => ['custom_term_cleanup/select_all']],
    ];

    $form['terms'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Vocabulary'),
        $this->t('Term Name'),
        $this->t('Usage Count'),
        $this->t('Delete'),
        $this->t('Select'),
      ],
    ];

    $vocabularies = Vocabulary::loadMultiple();
    $row_index = 0;

    foreach ($vocabularies as $vocabulary) {
      $vid = $vocabulary->id();
      $vocabulary_name = $vocabulary->label();
      $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree($vid);

      foreach ($terms as $term) {
        $term_id = $term->tid;
        $usage_count = $this->getTermUsageCount($term_id);

        if ($usage_count < 10) {
          $form['terms'][$row_index]['vocabulary'] = ['#markup' => $vocabulary_name];
          $form['terms'][$row_index]['name'] = ['#markup' => $term->name];
          $form['terms'][$row_index]['usage'] = ['#markup' => $usage_count];
          $form['terms'][$row_index]['delete'] = [
            '#type' => 'link',
            '#title' => $this->t('Delete'),
            '#url' => Url::fromRoute('custom_term_cleanup.confirm_delete', ['term_id' => $term_id]),
          ];
          $form['terms'][$row_index]['select'] = [
            '#type' => 'checkbox',
            '#value' => $term_id,
            '#name' => 'select[' . $term_id . ']',
            '#disabled' => TRUE,
          ];
          $row_index++;
        }
      }
    }

    $form['actions'] = [
      '#type' => 'actions',
      'delete_selected' => [
        '#type' => 'submit',
        '#value' => $this->t('Delete Selected'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $select_all = $form_state->getValue('select_all');
    $terms = $form_state->getValue('terms');
    $selected_terms = [];

    if ($select_all) {
      // If 'Select All' is checked, delete all terms with usage count < 10.
      foreach ($terms as $term) {
        if (isset($term['select']) && !empty($term['select'])) {
          $selected_terms[] = $term['select'];
        }
      }
      $selected_terms = array_unique($selected_terms);
    }
    else {
      foreach ($terms as $term) {
        if (isset($term['select']) && !empty($term['select'])) {
          $selected_terms[] = $term['select'];
        }
      }
      $selected_terms = array_unique($selected_terms);
    }

    if (!empty($selected_terms)) {
      foreach ($selected_terms as $term_id) {
        $term = Term::load($term_id);
        if ($term) {
          $term->delete();
          $this->messenger()->addMessage($this->t('The term "@name" has been deleted.', ['@name' => $term->getName()]));
        }
      }
    }
    else {
      $this->messenger()->addWarning($this->t('No terms selected for deletion.'));
    }

    $form_state->setRedirect('custom_term_cleanup.list_and_confirm_terms');
  }

  /**
   * Get the usage count of a taxonomy term.
   *
   * @param int $term_id
   *   The taxonomy term ID.
   *
   * @return int
   *   The number of times the term is used in content.
   */
  protected function getTermUsageCount($term_id) {
    $query = $this->database->select('taxonomy_index', 'ti');
    $query->addExpression('COUNT(ti.tid)', 'count');
    $query->condition('ti.tid', $term_id);

    return (int) $query->execute()->fetchField();
  }

}
